@@ -501,7 +501,8 @@ and __ocaml_lex_token_rec lexbuf __ocaml_lex_state =
                  ( END )
 # 503 "lexer.ml"
 
-  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_token_rec lexbuf __ocaml_lex_state
+  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; 
+      __ocaml_lex_token_rec lexbuf __ocaml_lex_state
 
 ;;
 
