@@ -1,6 +1,6 @@
-(* $Id: omega_SYM.ml 6004 2014-07-09 15:46:54Z cweiss $
+(* $Id: omega_SYM.ml 6465 2015-01-10 15:22:31Z jr_reuter $
 
-   Copyright (C) 1999-2014 by
+   Copyright (C) 1999-2015 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -25,9 +25,9 @@
 
 let rcs_file = RCS.parse "omega_SYM"
     ["Super Yang-Mills (incomplete, just for stress-testing Colorize.It())"]
-    { RCS.revision = "$Revision: 6004 $";
-      RCS.date = "$Date: 2014-07-09 17:46:54 +0200 (Wed, 09 Jul 2014) $";
-      RCS.author = "$Author: cweiss $";
+    { RCS.revision = "$Revision: 6465 $";
+      RCS.date = "$Date: 2015-01-10 16:22:31 +0100 (Sat, 10 Jan 2015) $";
+      RCS.author = "$Author: jr_reuter $";
       RCS.source
         = "$URL: svn+ssh://jr_reuter@login.hepforge.org/hepforge/svn/whizard/trunk/omega/src/omega_SYM.ml $" }
 
