@@ -1,4 +1,4 @@
-! WHIZARD 2.2.6 May 02 2015
+! WHIZARD 2.2.7 Aug 11 2015
 ! 
 ! Copyright (C) 1999-2015 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
@@ -37,9 +37,8 @@ module sf_circe1
   use iso_varying_string, string_t => varying_string
   use io_units
   use format_defs, only: FMT_17, FMT_19
-  use unit_tests
   use diagnostics
-  use physics_defs, only: UNDEFINED, ELECTRON, PHOTON
+  use physics_defs, only: ELECTRON, PHOTON
   use lorentz
   use rng_base
   use pdg_arrays
@@ -49,9 +48,7 @@ module sf_circe1
   use quantum_numbers
   use state_matrices
   use polarizations
-  use interactions
   use sf_mappings
-  use sf_aux
   use sf_base
   use circe1, circe1_rng_t => rng_type !NODEP!
 
@@ -59,7 +56,7 @@ module sf_circe1
   private
 
   public :: circe1_data_t 
-  public :: sf_circe1_test
+  public :: circe1_t
 
   type, extends (sf_data_t) :: circe1_data_t 
      private 
@@ -564,292 +561,4 @@ contains
   end function peak
   
 
-  subroutine sf_circe1_test (u, results)
-    integer, intent(in) :: u
-    type(test_results_t), intent(inout) :: results
-    call test (sf_circe1_1, "sf_circe1_1", &
-         "structure function configuration", &
-         u, results)
-    call test (sf_circe1_2, "sf_circe1_2", &
-         "structure function instance", &
-         u, results)
-    call test (sf_circe1_3, "sf_circe1_3", &
-         "generator mode", &
-         u, results)
-  end subroutine sf_circe1_test
-  
-  subroutine sf_circe1_1 (u)
-    integer, intent(in) :: u
-    type(model_data_t), target :: model
-    type(pdg_array_t), dimension(2) :: pdg_in
-    type(pdg_array_t), dimension(2) :: pdg_out
-    integer, dimension(:), allocatable :: pdg1, pdg2
-    class(sf_data_t), allocatable :: data
-    
-    write (u, "(A)")  "* Test output: sf_circe1_1"
-    write (u, "(A)")  "*   Purpose: initialize and display &
-         &CIRCE structure function data"
-    write (u, "(A)")
-    
-    write (u, "(A)")  "* Create empty data object"
-    write (u, "(A)")
-
-    call model%init_qed_test ()
-    pdg_in(1) = ELECTRON
-    pdg_in(2) = -ELECTRON
-
-    allocate (circe1_data_t :: data)
-    call data%write (u)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Initialize"
-    write (u, "(A)")
-
-    select type (data)
-    type is (circe1_data_t)
-       call data%init (model, pdg_in, &
-            sqrts = 500._default, &
-            eps = 1e-6_default, &
-            out_photon = [.false., .false.], &
-            ver = 0, &
-            rev = 0, &
-            acc = "SBAND", &
-            chat = 0, &
-            with_radiation = .true.)
-    end select
-
-    call data%write (u)
-
-    write (u, "(A)")
-
-    write (u, "(1x,A)")  "Outgoing particle codes:"
-    call data%get_pdg_out (pdg_out)
-    pdg1 = pdg_out(1)
-    pdg2 = pdg_out(2)
-    write (u, "(2x,99(1x,I0))")  pdg1, pdg2
-
-    call model%final ()
-    
-    write (u, "(A)")
-    write (u, "(A)")  "* Test output end: sf_circe1_1"
-
-  end subroutine sf_circe1_1
-
-  subroutine sf_circe1_2 (u)
-    integer, intent(in) :: u
-    type(model_data_t), target :: model
-    type(flavor_t), dimension(2) :: flv
-    type(pdg_array_t), dimension(2) :: pdg_in
-    class(sf_data_t), allocatable, target :: data
-    class(sf_int_t), allocatable :: sf_int
-    type(vector4_t) :: k1, k2
-    type(vector4_t), dimension(4) :: q
-    real(default) :: E
-    real(default), dimension(:), allocatable :: r, rb, x
-    real(default) :: f
-    
-    write (u, "(A)")  "* Test output: sf_circe1_2"
-    write (u, "(A)")  "*   Purpose: initialize and fill &
-         &circe1 structure function object"
-    write (u, "(A)")
-    
-    write (u, "(A)")  "* Initialize configuration data"
-    write (u, "(A)")
-
-    call model%init_qed_test ()
-    call flv(1)%init (ELECTRON, model)
-    call flv(2)%init (-ELECTRON, model)
-    pdg_in(1) = ELECTRON
-    pdg_in(2) = -ELECTRON
-
-    call reset_interaction_counter ()
-    
-    allocate (circe1_data_t :: data)
-    select type (data)
-    type is (circe1_data_t)
-       call data%init (model, pdg_in, &
-            sqrts = 500._default, &
-            eps = 1e-6_default, &
-            out_photon = [.false., .false.], &
-            ver = 0, &
-            rev = 0, &
-            acc = "SBAND", &
-            chat = 0, &
-            with_radiation = .true.)
-    end select
-       
-    write (u, "(A)")  "* Initialize structure-function object"
-    write (u, "(A)")
-    
-    call data%allocate_sf_int (sf_int)
-    call sf_int%init (data)
-    call sf_int%set_beam_index ([1,2])
-    
-    call sf_int%write (u)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Initialize incoming momentum with E=500"
-    write (u, "(A)")
-    E = 250
-    k1 = vector4_moving (E, sqrt (E**2 - flv(1)%get_mass ()**2), 3)
-    k2 = vector4_moving (E,-sqrt (E**2 - flv(2)%get_mass ()**2), 3)
-    call vector4_write (k1, u)
-    call vector4_write (k2, u)
-    call sf_int%seed_kinematics ([k1, k2])
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Set kinematics for x=0.95,0.85."
-    write (u, "(A)")
-
-    allocate (r (data%get_n_par ()))
-    allocate (rb(size (r)))
-    allocate (x (size (r)))
-
-    r = [0.9_default, 0.8_default]
-    call sf_int%complete_kinematics (x, f, r, rb, map=.false.)
-    call sf_int%write (u)
-
-    write (u, "(A)")
-    write (u, "(A,9(1x,F10.7))")  "x =", x
-    write (u, "(A,9(1x,F10.7))")  "f =", f
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Recover x from momenta"
-    write (u, "(A)")
-
-    q = sf_int%get_momenta (outgoing=.true.)
-    call sf_int%final ()
-    deallocate (sf_int)
-
-    call data%allocate_sf_int (sf_int)
-    call sf_int%init (data)
-    call sf_int%set_beam_index ([1, 2])
-
-    call sf_int%seed_kinematics ([k1, k2])
-    call sf_int%set_momenta (q, outgoing=.true.)
-    call sf_int%recover_x (x)
-
-    write (u, "(A,9(1x,F10.7))")  "x =", x
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Evaluate"
-    write (u, "(A)")
-
-    call sf_int%complete_kinematics (x, f, r, rb, map=.false.) 
-    call sf_int%apply (scale = 0._default)
-    call sf_int%write (u)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Cleanup"
-
-    call sf_int%final ()
-    call model%final ()
-    
-    write (u, "(A)")
-    write (u, "(A)")  "* Test output end: sf_circe1_2"
-
-  end subroutine sf_circe1_2
-
-  subroutine sf_circe1_3 (u)
-    integer, intent(in) :: u
-    type(model_data_t), target :: model
-    type(flavor_t), dimension(2) :: flv
-    type(pdg_array_t), dimension(2) :: pdg_in
-    class(sf_data_t), allocatable, target :: data
-    class(rng_factory_t), allocatable :: rng_factory
-    class(sf_int_t), allocatable :: sf_int
-    type(vector4_t) :: k1, k2
-    real(default) :: E
-    real(default), dimension(:), allocatable :: r, rb, x
-    real(default) :: f, x_free
-    
-    write (u, "(A)")  "* Test output: sf_circe1_3"
-    write (u, "(A)")  "*   Purpose: initialize and fill &
-         &circe1 structure function object"
-    write (u, "(A)")
-    
-    write (u, "(A)")  "* Initialize configuration data"
-    write (u, "(A)")
-
-    call model%init_qed_test ()
-    call flv(1)%init (ELECTRON, model)
-    call flv(2)%init (-ELECTRON, model)
-    pdg_in(1) = ELECTRON
-    pdg_in(2) = -ELECTRON
-
-    call reset_interaction_counter ()
-    
-    allocate (circe1_data_t :: data)
-    allocate (rng_test_factory_t :: rng_factory)
-    select type (data)
-    type is (circe1_data_t)
-       call data%init (model, pdg_in, &
-            sqrts = 500._default, &
-            eps = 1e-6_default, &
-            out_photon = [.false., .false.], &
-            ver = 0, &
-            rev = 0, &
-            acc = "SBAND", &
-            chat = 0, &
-            with_radiation = .true.)
-       call data%set_generator_mode (rng_factory)
-    end select
-       
-    write (u, "(A)")  "* Initialize structure-function object"
-    write (u, "(A)")
-    
-    call data%allocate_sf_int (sf_int)
-    call sf_int%init (data)
-    call sf_int%set_beam_index ([1,2])
-    select type (sf_int)
-    type is (circe1_t)
-       call sf_int%rng_obj%rng%init (3)
-    end select
-
-    write (u, "(A)")  "* Initialize incoming momentum with E=500"
-    write (u, "(A)")
-    E = 250
-    k1 = vector4_moving (E, sqrt (E**2 - flv(1)%get_mass ()**2), 3)
-    k2 = vector4_moving (E,-sqrt (E**2 - flv(2)%get_mass ()**2), 3)
-    call vector4_write (k1, u)
-    call vector4_write (k2, u)
-    call sf_int%seed_kinematics ([k1, k2])
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Generate x"
-    write (u, "(A)")
-
-    allocate (r (data%get_n_par ()))
-    allocate (rb(size (r)))
-    allocate (x (size (r)))
-
-    r  = 0
-    rb = 0
-    x_free = 1
-    call sf_int%generate_free (r, rb, x_free)
-    call sf_int%complete_kinematics (x, f, r, rb, map=.false.)
-
-    write (u, "(A,9(1x,F10.7))")  "x =", x
-    write (u, "(A,9(1x,F10.7))")  "f =", f
-    write (u, "(A,9(1x,F10.7))")  "xf=", x_free
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Evaluate"
-    write (u, "(A)")
-
-    call sf_int%apply (scale = 0._default)
-    call sf_int%write (u)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Cleanup"
-
-    call sf_int%final ()
-    call model%final ()
-    
-    write (u, "(A)")
-    write (u, "(A)")  "* Test output end: sf_circe1_3"
-
-  end subroutine sf_circe1_3
-
-
 end module sf_circe1
