@@ -1,4 +1,4 @@
-! WHIZARD 2.2.6 May 02 2015
+! WHIZARD 2.2.7 Aug 11 2015
 ! 
 ! Copyright (C) 1999-2015 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
@@ -36,27 +36,21 @@ module sf_escan
   use iso_varying_string, string_t => varying_string
   use io_units
   use format_defs, only: FMT_12
-  use unit_tests
+  use unit_tests, only: nearly_equal
   use diagnostics
   use lorentz
-  use physics_defs, only: ELECTRON
   use pdg_arrays
   use model_data
   use flavors
-  use helicities
-  use colors
   use quantum_numbers
   use state_matrices
   use polarizations
-  use interactions
-  use sf_aux
   use sf_base
   
   implicit none
   private
 
   public :: escan_data_t
-  public :: sf_escan_test
 
   type, extends(sf_data_t) :: escan_data_t
      private
@@ -102,7 +96,7 @@ contains
     end do
     m2 = data%flv_in(1,:)%get_mass ()
     do i = 1, 2
-       if (any (data%flv_in(1:data%n_flv(i),i)%get_mass () /= m2(i))) then
+       if (.not. any (nearly_equal (data%flv_in(1:data%n_flv(i),i)%get_mass (), m2(i)))) then
           call msg_fatal ("Energy scan: incoming particle mass must be uniform")
        end if
     end do
@@ -299,154 +293,4 @@ contains
   end subroutine escan_apply
 
 
-  subroutine sf_escan_test (u, results)
-    integer, intent(in) :: u
-    type(test_results_t), intent(inout) :: results
-    call test (sf_escan_1, "sf_escan_1", &
-         "structure function configuration", &
-         u, results)
-    call test (sf_escan_2, "sf_escan_2", &
-         "generate event", &
-         u, results)
-  end subroutine sf_escan_test
-  
-  subroutine sf_escan_1 (u)
-    integer, intent(in) :: u
-    type(model_data_t), target :: model
-    type(pdg_array_t), dimension(2) :: pdg_in
-    type(pdg_array_t), dimension(2) :: pdg_out
-    integer, dimension(:), allocatable :: pdg1, pdg2
-    class(sf_data_t), allocatable :: data
-    
-    write (u, "(A)")  "* Test output: sf_escan_1"
-    write (u, "(A)")  "*   Purpose: initialize and display &
-         &energy-scan structure function data"
-    write (u, "(A)")
-    
-    call model%init_qed_test ()
-    pdg_in(1) = ELECTRON
-    pdg_in(2) = -ELECTRON
-
-    allocate (escan_data_t :: data)
-    select type (data)
-    type is (escan_data_t)
-       call data%init (model, pdg_in, norm = 2._default)
-    end select
-
-    call data%write (u)
-
-    write (u, "(A)")
-
-    write (u, "(1x,A)")  "Outgoing particle codes:"
-    call data%get_pdg_out (pdg_out)
-    pdg1 = pdg_out(1)
-    pdg2 = pdg_out(2)
-    write (u, "(2x,99(1x,I0))")  pdg1, pdg2
-
-    call model%final ()
-    
-    write (u, "(A)")
-    write (u, "(A)")  "* Test output end: sf_escan_1"
-
-  end subroutine sf_escan_1
-
-  subroutine sf_escan_2 (u)
-    integer, intent(in) :: u
-    type(model_data_t), target :: model
-    type(flavor_t), dimension(2) :: flv
-    type(pdg_array_t), dimension(2) :: pdg_in
-    class(sf_data_t), allocatable, target :: data
-    class(sf_int_t), allocatable :: sf_int
-    type(vector4_t) :: k1, k2
-    real(default) :: E
-    real(default), dimension(:), allocatable :: r, rb, x
-    real(default) :: x_free, f
-    
-    write (u, "(A)")  "* Test output: sf_escan_2"
-    write (u, "(A)")  "*   Purpose: initialize and display &
-         &beam-events structure function data"
-    write (u, "(A)")
-    
-    call model%init_qed_test ()
-    call flv(1)%init (ELECTRON, model)
-    call flv(2)%init (-ELECTRON, model)
-    pdg_in(1) = ELECTRON
-    pdg_in(2) = -ELECTRON
-
-    call reset_interaction_counter ()
-    
-    allocate (escan_data_t :: data)
-    select type (data)
-    type is (escan_data_t)
-       call data%init (model, pdg_in)
-    end select
-       
-    write (u, "(A)")  "* Initialize structure-function object"
-    write (u, "(A)")
-    
-    call data%allocate_sf_int (sf_int)
-    call sf_int%init (data)
-    call sf_int%set_beam_index ([1,2])
-
-    write (u, "(A)")  "* Initialize incoming momentum with E=500"
-    write (u, "(A)")
-    E = 250
-    k1 = vector4_moving (E, sqrt (E**2 - flv(1)%get_mass ()**2), 3)
-    k2 = vector4_moving (E,-sqrt (E**2 - flv(2)%get_mass ()**2), 3)
-    call vector4_write (k1, u)
-    call vector4_write (k2, u)
-    call sf_int%seed_kinematics ([k1, k2])
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Set dummy parameters and generate x"
-    write (u, "(A)")
-
-    allocate (r (data%get_n_par ()))
-    allocate (rb(size (r)))
-    allocate (x (size (r)))
-
-    r  = 0.8
-    rb = 1 - r
-    x_free = 1
-
-    call sf_int%complete_kinematics (x, f, r, rb, map=.false.)
-
-    write (u, "(A,9(1x,F10.7))")  "r =", r
-    write (u, "(A,9(1x,F10.7))")  "rb=", rb
-    write (u, "(A,9(1x,F10.7))")  "x =", x
-    write (u, "(A,9(1x,F10.7))")  "f =", f
-    write (u, "(A,9(1x,F10.7))")  "xf=", x_free
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Inverse kinematics"
-    write (u, "(A)")
-
-    call sf_int%recover_x (x, x_free)
-    call sf_int%inverse_kinematics (x, f, r, rb, map=.false.)
-
-    write (u, "(A,9(1x,F10.7))")  "r =", r
-    write (u, "(A,9(1x,F10.7))")  "rb=", rb
-    write (u, "(A,9(1x,F10.7))")  "x =", x
-    write (u, "(A,9(1x,F10.7))")  "f =", f
-    write (u, "(A,9(1x,F10.7))")  "xf=", x_free
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Evaluate"
-    write (u, "(A)")
-
-    call sf_int%apply (scale = 0._default)
-    call sf_int%write (u)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Cleanup"
-
-    call sf_int%final ()
-    call model%final ()
-    
-    write (u, "(A)")
-    write (u, "(A)")  "* Test output end: sf_escan_2"
-
-  end subroutine sf_escan_2
-
-
 end module sf_escan
