@@ -1,4 +1,4 @@
-! WHIZARD 2.2.6 May 02 2015
+! WHIZARD 2.2.7 Aug 11 2015
 ! 
 ! Copyright (C) 1999-2015 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
@@ -36,7 +36,6 @@ module sf_lhapdf
   use iso_varying_string, string_t => varying_string
   use format_defs, only: FMT_17, FMT_19
   use io_units
-  use unit_tests
   use system_dependencies, only: LHAPDF_PDFSETS_PATH
   use system_dependencies, only: LHAPDF5_AVAILABLE
   use system_dependencies, only: LHAPDF6_AVAILABLE  
@@ -54,8 +53,6 @@ module sf_lhapdf
   use quantum_numbers
   use state_matrices
   use polarizations
-  use interactions
-  use sf_aux
   use sf_base
   use lhapdf !NODEP!
   use hoppet_interface
@@ -67,7 +64,6 @@ module sf_lhapdf
   public :: lhapdf_initialize
   public :: lhapdf_data_t
   public :: alpha_qcd_lhapdf_t
-  public :: sf_lhapdf_test
 
   type :: lhapdf_global_status_t
      private
@@ -132,7 +128,7 @@ module sf_lhapdf
   character(*), parameter :: LHAPDF5_DEFAULT_PHOTON = "GSG960.LHgrid"
   character(*), parameter :: LHAPDF6_DEFAULT_PROTON = "CT10"
 
-  type(lhapdf_global_status_t) :: lhapdf_global_status
+  type(lhapdf_global_status_t), save :: lhapdf_global_status
 
   interface
      subroutine InitPDFsetM (set, file)
@@ -276,8 +272,10 @@ contains
        end if
        call InitPDFM (set, member)
     else if (LHAPDF6_AVAILABLE) then
-       if (lhapdf_global_status_is_initialized (set) .and. &
-            pdf%is_associated ()) return
+    ! TODO: (bcn 2015-07-07) we should have a closer look why this global 
+    !                        check must not be executed
+    !   if (lhapdf_global_status_is_initialized (set) .and. &
+    !        pdf%is_associated ()) return
        if (file == "") then
           select case (set)
           case (1);  file = LHAPDF6_DEFAULT_PROTON
@@ -620,10 +618,10 @@ contains
       sf_int%q = scale
       x = sf_int%x
       s = sf_int%s
+      xx = x
       qq = min (data% qmax, scale)
       qq = max (data% qmin, qq)
       if (.not. data% photon) then
-         xx = x
          if (data% invert) then
             if (data%has_photon) then
                if (LHAPDF5_AVAILABLE) then
@@ -726,239 +724,4 @@ contains
   end subroutine alpha_qcd_lhapdf_init
     
 
-  subroutine sf_lhapdf_test (u, results)
-    integer, intent(in) :: u
-    type(test_results_t), intent(inout) :: results
-    if (LHAPDF5_AVAILABLE) then  
-       call test (sf_lhapdf_1, "sf_lhapdf5_1", &
-            "structure function configuration", &
-            u, results)
-    else if (LHAPDF6_AVAILABLE) then
-       call test (sf_lhapdf_1, "sf_lhapdf6_1", &
-            "structure function configuration", &
-            u, results)
-    end if
-    if (LHAPDF5_AVAILABLE) then
-       call test (sf_lhapdf_2, "sf_lhapdf5_2", &
-            "structure function instance", &
-            u, results)
-    else if (LHAPDF6_AVAILABLE) then
-       call test (sf_lhapdf_2, "sf_lhapdf6_2", &
-            "structure function instance", &
-            u, results)     
-    end if
-    if (LHAPDF5_AVAILABLE) then
-       call test (sf_lhapdf_3, "sf_lhapdf5_3", &
-            "running alpha_s", &
-            u, results)
-    else if (LHAPDF6_AVAILABLE) then
-       call test (sf_lhapdf_3, "sf_lhapdf6_3", &
-            "running alpha_s", &
-            u, results)     
-    end if
-  end subroutine sf_lhapdf_test
-  
-  subroutine sf_lhapdf_1 (u)
-    integer, intent(in) :: u
-    type(model_data_t), target :: model
-    type(pdg_array_t) :: pdg_in
-    type(pdg_array_t), dimension(1) :: pdg_out
-    integer, dimension(:), allocatable :: pdg1
-    class(sf_data_t), allocatable :: data
-    
-    write (u, "(A)")  "* Test output: sf_lhapdf_1"
-    write (u, "(A)")  "*   Purpose: initialize and display &
-         &test structure function data"
-    write (u, "(A)")
-    
-    write (u, "(A)")  "* Create empty data object"
-    write (u, "(A)")
-
-    call model%init_sm_test ()
-    pdg_in = PROTON
-
-    allocate (lhapdf_data_t :: data)
-    call data%write (u)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Initialize"
-    write (u, "(A)")
-
-    select type (data)
-    type is (lhapdf_data_t)
-       call data%init (model, pdg_in)
-    end select
-
-    call data%write (u)
-
-    write (u, "(A)")
-
-    write (u, "(1x,A)")  "Outgoing particle codes:"
-    call data%get_pdg_out (pdg_out)
-    pdg1 = pdg_out(1)
-    write (u, "(2x,99(1x,I0))")  pdg1
-
-    call model%final ()
-    
-    write (u, "(A)")
-    write (u, "(A)")  "* Test output end: sf_lhapdf_1"
-
-  end subroutine sf_lhapdf_1
-
-  subroutine sf_lhapdf_2 (u)
-    integer, intent(in) :: u
-    type(model_data_t), target :: model
-    type(flavor_t) :: flv
-    type(pdg_array_t) :: pdg_in
-    class(sf_data_t), allocatable, target :: data
-    class(sf_int_t), allocatable :: sf_int
-    type(vector4_t) :: k
-    type(vector4_t), dimension(2) :: q
-    real(default) :: E
-    real(default), dimension(:), allocatable :: r, rb, x
-    real(default) :: f
-    
-    write (u, "(A)")  "* Test output: sf_lhapdf_2"
-    write (u, "(A)")  "*   Purpose: initialize and fill &
-         &test structure function object"
-    write (u, "(A)")
-    
-    write (u, "(A)")  "* Initialize configuration data"
-    write (u, "(A)")
-
-    call model%init_sm_test ()
-    call flv%init (PROTON, model)
-    pdg_in = PROTON
-    call lhapdf_global_reset ()
-    
-    call reset_interaction_counter ()
-    
-    allocate (lhapdf_data_t :: data)
-    select type (data)
-    type is (lhapdf_data_t)
-       call data%init (model, pdg_in)
-    end select
-       
-    write (u, "(A)")  "* Initialize structure-function object"
-    write (u, "(A)")
-    
-    call data%allocate_sf_int (sf_int)
-    call sf_int%init (data)
-    call sf_int%set_beam_index ([1])
-    
-    call sf_int%write (u)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Initialize incoming momentum with E=500"
-    write (u, "(A)")
-    E = 500
-    k = vector4_moving (E, sqrt (E**2 - flv%get_mass ()**2), 3)
-    call vector4_write (k, u)
-    call sf_int%seed_kinematics ([k])
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Set kinematics for x=0.5"
-    write (u, "(A)")
-
-    allocate (r (data%get_n_par ()))
-    allocate (rb(size (r)))
-    allocate (x (size (r)))
-
-    r = 0.5_default
-    rb = 1 - r
-    call sf_int%complete_kinematics (x, f, r, rb, map=.false.)
-    call sf_int%write (u)
-
-    write (u, "(A)")
-    write (u, "(A,9(1x,F10.7))")  "x =", x
-    write (u, "(A,9(1x,F10.7))")  "f =", f
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Recover x from momenta"
-    write (u, "(A)")
-
-    q = sf_int%get_momenta (outgoing=.true.)
-    call sf_int%final ()
-    deallocate (sf_int)
-
-    call data%allocate_sf_int (sf_int)
-    call sf_int%init (data)
-    call sf_int%set_beam_index ([1])
-
-    call sf_int%seed_kinematics ([k])
-    call sf_int%set_momenta (q, outgoing=.true.)
-    call sf_int%recover_x (x)
-
-    write (u, "(A,9(1x,F10.7))")  "x =", x
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Evaluate for Q = 100 GeV"
-    write (u, "(A)")
-
-    call sf_int%complete_kinematics (x, f, r, rb, map=.false.) 
-    call sf_int%apply (scale = 100._default)
-    call sf_int%write (u, testflag = .true.)
-
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Cleanup"
-
-    call sf_int%final ()
-    call model%final ()
-    
-    write (u, "(A)")
-    write (u, "(A)")  "* Test output end: sf_lhapdf_2"
-
-  end subroutine sf_lhapdf_2
-
-  subroutine sf_lhapdf_3 (u)
-    integer, intent(in) :: u
-    type(qcd_t) :: qcd
-    type(string_t) :: name, path
-    integer :: member
-    
-    write (u, "(A)")  "* Test output: sf_lhapdf_3"
-    write (u, "(A)")  "*   Purpose: initialize and evaluate alpha_s"
-    write (u, "(A)")
-    
-    write (u, "(A)")  "* Initialize configuration data"
-    write (u, "(A)")
-
-    call lhapdf_global_reset ()
-
-    if (LHAPDF5_AVAILABLE) then
-       name = "cteq6ll.LHpdf"
-       member = 1
-       path = ""
-    else if (LHAPDF6_AVAILABLE) then
-       name = "CT10"
-       member = 1
-       path = ""
-    end if
-       
-    write (u, "(A)")  "* Initialize qcd object"
-    write (u, "(A)")
-    
-    allocate (alpha_qcd_lhapdf_t :: qcd%alpha)
-    select type (alpha => qcd%alpha)
-    type is (alpha_qcd_lhapdf_t)
-       call alpha%init (name, member, path)
-    end select
-    call qcd%write (u)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Evaluate for Q = 100"
-    write (u, "(A)")
-    
-    write (u, "(1x,A,F8.5)")  "alpha = ", qcd%alpha%get (100._default)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Cleanup"
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Test output end: sf_lhapdf_3"
-
-  end subroutine sf_lhapdf_3
-
-
 end module sf_lhapdf
