@@ -1,4 +1,4 @@
-! WHIZARD 2.2.6 May 02 2015
+! WHIZARD 2.2.7 Aug 11 2015
 ! 
 ! Copyright (C) 1999-2015 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
@@ -32,17 +32,11 @@
 
 module eio_hepmc
   
-  use kinds
-  use io_units
   use iso_varying_string, string_t => varying_string
+  use io_units
   use string_utils
-  use unit_tests
   use diagnostics
-  use os_interface
-  use lorentz
-  use model_data
   use particles
-  use subevents
   use event_base
   use hep_events
   use eio_data
@@ -53,15 +47,16 @@ module eio_hepmc
   private
 
   public :: eio_hepmc_t
-  public :: eio_hepmc_test
 
   type, extends (eio_t) :: eio_hepmc_t
      logical :: writing = .false.
      logical :: reading = .false.
+     type(event_sample_data_t) :: data
      ! logical :: keep_beams = .false.
      logical :: recover_beams = .false.
      logical :: use_alpha_s_from_file = .false.
      logical :: use_scale_from_file = .false.
+     logical :: output_cross_section = .false.
      type(hepmc_iostream_t) :: iostream
      type(hepmc_event_t) :: hepmc_event
      integer, dimension(:), allocatable :: proc_num_id
@@ -77,21 +72,21 @@ module eio_hepmc
      procedure :: output => eio_hepmc_output
      procedure :: input_i_prc => eio_hepmc_input_i_prc
      procedure :: input_event => eio_hepmc_input_event
+     procedure :: skip => eio_hepmc_skip
   end type eio_hepmc_t
   
 
-
-
 contains
   
   ! subroutine eio_hepmc_set_parameters (eio, keep_beams, recover_beams, extension)
   subroutine eio_hepmc_set_parameters &
        (eio, recover_beams, use_alpha_s_from_file, use_scale_from_file, &
-       extension)
+       extension, output_cross_section)
     class(eio_hepmc_t), intent(inout) :: eio
     logical, intent(in), optional :: recover_beams 
     logical, intent(in), optional :: use_alpha_s_from_file
     logical, intent(in), optional :: use_scale_from_file
+    logical, intent(in), optional :: output_cross_section
     type(string_t), intent(in), optional :: extension    
     if (present (recover_beams))  eio%recover_beams = recover_beams
     if (present (use_alpha_s_from_file)) &
@@ -103,6 +98,8 @@ contains
     else
        eio%extension = "hepmc"
     end if
+    if (present (output_cross_section)) &
+         eio%output_cross_section = output_cross_section
   end subroutine eio_hepmc_set_parameters
   
   subroutine eio_hepmc_write (object, unit)
@@ -171,16 +168,16 @@ contains
     type(event_sample_data_t), intent(in), optional :: data
     if (.not. present (data)) &
          call msg_bug ("HepMC initialization: missing data")
+    eio%data = data
     if (data%n_beam /= 2) &
          call msg_fatal ("HepMC: defined for scattering processes only")    
+    ! We could relax this condition now with weighted hepmc events
     if (data%unweighted) then
        select case (data%norm_mode)
        case (NORM_UNIT)
        case default; call msg_fatal &
             ("HepMC: normalization for unweighted events must be '1'")
        end select
-    else
-       call msg_fatal ("HepMC: events must be unweighted")    
     end if
     eio%sample = sample    
     if (present (extension)) then
@@ -196,7 +193,7 @@ contains
     type(string_t), intent(in), optional :: extension
     type(event_sample_data_t), intent(in), optional :: data
     logical, intent(out), optional :: success
-    call eio%set_splitting (data)    
+    call eio%set_splitting (data)
     call eio%common_init (sample, data, extension)
     write (msg_buffer, "(A,A,A)")  "Events: writing to HepMC file '", &
          char (eio%filename), "'"
@@ -246,9 +243,16 @@ contains
        call hepmc_event_init (eio%hepmc_event, &
             proc_id = eio%proc_num_id (i_prc), &
             event_id = event%get_index ())
-       call hepmc_event_from_particle_set (eio%hepmc_event, pset_ptr)
+       if (eio%output_cross_section) then
+          call hepmc_event_from_particle_set (eio%hepmc_event, pset_ptr, &
+               eio%data%cross_section(i_prc), eio%data%error(i_prc))
+       else 
+          call hepmc_event_from_particle_set (eio%hepmc_event, pset_ptr)
+       end if
        call hepmc_event_set_scale (eio%hepmc_event, event%get_fac_scale ())
        call hepmc_event_set_alpha_qcd (eio%hepmc_event, event%get_alpha_s ())
+       if (.not. eio%data%unweighted) &
+          call hepmc_event_add_weight (eio%hepmc_event, event%weight_prc)
        call hepmc_iostream_write_event (eio%iostream, eio%hepmc_event)
        call hepmc_event_final (eio%hepmc_event)
     else
@@ -291,8 +295,6 @@ contains
     class(eio_hepmc_t), intent(inout) :: eio
     class(generic_event_t), intent(inout), target :: event
     integer, intent(out) :: iostat
-    type(particle_set_t), pointer :: pset
-    logical :: ok    
     iostat = 0
     call event%reset ()
     call event%select (1, 1, 1)
@@ -303,237 +305,11 @@ contains
     call hepmc_event_final (eio%hepmc_event)
   end subroutine eio_hepmc_input_event
 
+  subroutine eio_hepmc_skip (eio, iostat)
+    class(eio_hepmc_t), intent(inout) :: eio
+    integer, intent(out) :: iostat
+    iostat = 0
+  end subroutine eio_hepmc_skip
 
-  subroutine eio_hepmc_test (u, results)
-    integer, intent(in) :: u
-    type(test_results_t), intent(inout) :: results
-    call test (eio_hepmc_1, "eio_hepmc_1", &
-         "write event contents", &
-         u, results)
-    call test (eio_hepmc_2, "eio_hepmc_2", &
-         "read event contents", &
-         u, results)
-  end subroutine eio_hepmc_test
-  
-  subroutine eio_hepmc_1 (u)
-    integer, intent(in) :: u
-    class(generic_event_t), pointer :: event
-    type(event_sample_data_t) :: data
-    class(eio_t), allocatable :: eio
-    type(string_t) :: sample
-    integer :: u_file, iostat
-    character(116) :: buffer
-
-    write (u, "(A)")  "* Test output: eio_hepmc_1"
-    write (u, "(A)")  "*   Purpose: write a HepMC file"
-    write (u, "(A)")
-
-    write (u, "(A)")  "* Initialize test process"
- 
-    call eio_prepare_test (event, unweighted=.false.)
-
-    call data%init (1)
-    data%n_beam = 2
-    data%unweighted = .true.
-    data%norm_mode = NORM_UNIT
-    data%pdg_beam = 25
-    data%energy_beam = 500
-    data%proc_num_id = [42]
-    data%cross_section(1) = 100
-    data%error(1) = 1
-    data%total_cross_section = sum (data%cross_section)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Generate and write an event"
-    write (u, "(A)")
- 
-    sample = "eio_hepmc_1"
- 
-    allocate (eio_hepmc_t :: eio)
-    select type (eio)
-    type is (eio_hepmc_t)
-       call eio%set_parameters ()
-    end select
-    
-    call eio%init_out (sample, data)
-    call event%generate (1, [0._default, 0._default])
-
-    call eio%output (event, i_prc = 1)
-    call eio%write (u)
-    call eio%final ()
-
-    write (u, "(A)")
-    write (u, "(A)")  "* File contents (blanking out last two digits):"
-    write (u, "(A)")
-
-    u_file = free_unit ()
-    open (u_file, file = char (sample // ".hepmc"), &
-         action = "read", status = "old")
-    do
-       read (u_file, "(A)", iostat = iostat)  buffer
-       if (iostat /= 0)  exit
-       if (trim (buffer) == "")  cycle
-       if (buffer(1:14) == "HepMC::Version")  cycle
-       if (buffer(1:10) == "P 10001 25") &
-            call buffer_blanker (buffer, 32, 55, 78)
-       if (buffer(1:10) == "P 10002 25") &
-            call buffer_blanker (buffer, 33, 56, 79)
-       if (buffer(1:10) == "P 10003 25") &
-            call buffer_blanker (buffer, 29, 53, 78, 101)
-       if (buffer(1:10) == "P 10004 25") &
-            call buffer_blanker (buffer, 28, 51, 76, 99)       
-       write (u, "(A)") trim (buffer)
-    end do
-    close (u_file)
-    
-    write (u, "(A)")
-    write (u, "(A)")  "* Reset data"
-    write (u, "(A)")
- 
-    deallocate (eio)
-    allocate (eio_hepmc_t :: eio)
-    
-    select type (eio)
-    type is (eio_hepmc_t)
-       call eio%set_parameters ()
-    end select
-    call eio%write (u)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Cleanup"
- 
-    call eio_cleanup_test (event)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Test output end: eio_hepmc_1"
-    
-  contains
-    
-    subroutine buffer_blanker (buf, pos1, pos2, pos3, pos4)
-      character(len=*), intent(inout) :: buf
-      integer, intent(in) :: pos1, pos2, pos3
-      integer, intent(in), optional :: pos4
-      type(string_t) :: line
-      line = var_str (trim (buf))
-      line = replace (line, pos1, "XX")
-      line = replace (line, pos2, "XX")
-      line = replace (line, pos3, "XX")
-      if (present (pos4)) then
-         line = replace (line, pos4, "XX")
-      end if
-      line = replace (line, "4999999999999", "5000000000000")
-      buf = char (line)
-    end subroutine buffer_blanker
-    
-  end subroutine eio_hepmc_1
-  
-  subroutine eio_hepmc_2 (u)
-    integer, intent(in) :: u
-    class(model_data_t), pointer :: fallback_model
-    class(generic_event_t), pointer :: event
-    type(event_sample_data_t) :: data
-    class(eio_t), allocatable :: eio
-    type(string_t) :: sample
-    integer :: u_file, iostat, i_prc
-
-    write (u, "(A)")  "* Test output: eio_hepmc_2"
-    write (u, "(A)")  "*   Purpose: read a HepMC event"
-    write (u, "(A)")
-
-    write (u, "(A)")  "* Write a HepMC data file"
-    write (u, "(A)")
-    
-    u_file = free_unit ()
-    sample = "eio_hepmc_2"
-    open (u_file, file = char (sample // ".hepmc"), &
-         status = "replace", action = "readwrite")
-    
-    write (u_file, "(A)")  "HepMC::Version 2.06.09"
-    write (u_file, "(A)")  "HepMC::IO_GenEvent-START_EVENT_LISTING"
-    write (u_file, "(A)")  "E 0 -1 -1.0000000000000000e+00 &
-         &-1.0000000000000000e+00 &
-         &-1.0000000000000000e+00 42 0 1 10001 10002 0 0"
-    write (u_file, "(A)")  "U GEV MM"
-    write (u_file, "(A)")  "V -1 0 0 0 0 0 2 2 0"
-    write (u_file, "(A)")  "P 10001 25 0 0 4.8412291827592713e+02 &
-         &5.0000000000000000e+02 &
-         &1.2499999999999989e+02 3 0 0 -1 0"
-    write (u_file, "(A)")  "P 10002 25 0 0 -4.8412291827592713e+02 &
-         &5.0000000000000000e+02 &
-         &1.2499999999999989e+02 3 0 0 -1 0"
-    write (u_file, "(A)")  "P 10003 25 -1.4960220911365536e+02 &
-         &-4.6042825611414656e+02 &
-         &0 5.0000000000000000e+02 1.2500000000000000e+02 1 0 0 0 0"
-    write (u_file, "(A)")  "P 10004 25 1.4960220911365536e+02 &
-         &4.6042825611414656e+02 &
-         &0 5.0000000000000000e+02 1.2500000000000000e+02 1 0 0 0 0"
-    write (u_file, "(A)")  "HepMC::IO_GenEvent-END_EVENT_LISTING"
-    close (u_file)
-
-    write (u, "(A)")  "* Initialize test process" 
-    write (u, "(A)")
-    
-    call eio_prepare_fallback_model (fallback_model)
-    call eio_prepare_test (event, unweighted=.false.)
-
-    allocate (eio_hepmc_t :: eio)
-    select type (eio)
-    type is (eio_hepmc_t)
-       call eio%set_parameters (recover_beams = .false.)
-    end select            
-    call eio%set_fallback_model (fallback_model)
-    
-    call data%init (1)
-    data%n_beam = 2
-    data%unweighted = .true.
-    data%norm_mode = NORM_UNIT
-    data%pdg_beam = 25
-    data%energy_beam = 500
-    data%proc_num_id = [42]
-    call data%write (u)
-
-    write (u, "(A)")    
-    write (u, "(A)")  "* Initialize"
-    write (u, "(A)")
-             
-    call eio%init_in (sample, data)
-    call eio%write (u)    
-    
-    write (u, "(A)")
-    write (u, "(A)")  "* Read event"
-    write (u, "(A)")
- 
-    call eio%input_i_prc (i_prc, iostat)
-
-    select type (eio)
-    type is (eio_hepmc_t)
-       write (u, "(A,I0,A,I0)")  "Found process #", i_prc, &
-            " with ID = ", eio%proc_num_id(i_prc)
-    end select
-    
-    call eio%input_event (event, iostat)
-    
-    call event%write (u)
-        
-    write (u, "(A)")
-    write (u, "(A)")  "* Read closing"
-    write (u, "(A)")
-    
-    call eio%input_i_prc (i_prc, iostat)
-    write (u, "(A,I0)")  "iostat = ", iostat
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Cleanup"
-
-    call eio%final ()
-
-    call eio_cleanup_test (event)
-    call eio_cleanup_fallback_model (fallback_model)
-
-    write (u, "(A)")
-    write (u, "(A)")  "* Test output end: eio_hepmc_2"
-    
-  end subroutine eio_hepmc_2
-  
 
 end module eio_hepmc
