@@ -1,4 +1,4 @@
-! WHIZARD 2.2.6 May 02 2015
+! WHIZARD 2.2.7 Aug 11 2015
 ! 
 ! Copyright (C) 1999-2015 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
@@ -35,11 +35,9 @@ module auto_components
   use kinds, only: default
   use iso_varying_string, string_t => varying_string
   use io_units
-  use unit_tests
   use diagnostics
   use model_data
   use pdg_arrays
-  use model_testbed
 
   implicit none
   private
@@ -54,8 +52,8 @@ module auto_components
   public :: constrain_in_state
   public :: ps_table_t
   public :: ds_table_t
+  public :: fs_table_t
   public :: if_table_t
-  public :: auto_components_test
 
   type, abstract :: split_constraint_t
    contains
@@ -156,6 +154,7 @@ module auto_components
      procedure :: record_sorted => ps_table_record_sorted
      procedure :: record => ps_table_record
      procedure :: get_length => ps_table_get_length
+     procedure :: get_emitters => ps_table_get_emitters
      procedure :: get_pdg_out => ps_table_get_pdg_out
   end type ps_table_t
      
@@ -532,7 +531,6 @@ contains
     integer, intent(in) :: index
     integer, intent(in) :: n_in
     type(string_t), intent(out), dimension(:), allocatable :: prt_in, prt_out
-    type(string_t) :: prt_tmp
     type(field_data_t), pointer :: prt
     type(ps_entry_t), pointer :: entry
     integer, dimension(:), allocatable :: pdg
@@ -632,7 +630,6 @@ contains
     type(split_constraints_t), intent(in) :: constraints
     type(pdg_list_t) :: pl_in
     type(pdg_list_t), dimension(0) :: pl
-    logical :: passed
     call table%init (model, pl, constraints)
     table%pdg_in = pdg_in
     call pl_in%init (1)
@@ -644,8 +641,6 @@ contains
     class(fs_table_t), intent(inout) :: table
     type(split_constraints_t) :: constraints
     type(ps_entry_t), pointer :: current
-    integer :: i, s
-    logical :: passed
     current => table%first
     do while (associated (current))
        call table%split (current, 0, constraints, record = .true.)
@@ -757,7 +752,6 @@ contains
     integer, intent(in) :: n_loop, n_rad
     type(split_constraints_t), intent(in) :: constraints
     logical, intent(out) :: passed
-    integer :: i
     call table%record (pl%sort_abs (2), n_loop, n_rad, constraints, passed)
   end subroutine if_table_record_sorted
 
@@ -768,7 +762,6 @@ contains
     type(split_constraints_t), intent(in) :: constraints
     logical, intent(out) :: passed
     type(ps_entry_t), pointer :: current
-    logical :: mass_limit
     if (.not. pl%is_regular ())  return
     call constraints%check_before_record (table, pl, n_loop, passed)
     if (.not. passed)  return
@@ -853,6 +846,40 @@ contains
     end do
   end function ps_table_get_length
 
+  function ps_table_get_emitters (table, constraints) result (emitters)
+     integer, dimension(:), allocatable :: emitters
+     class(ps_table_t), intent(in) :: table
+     type(split_constraints_t), intent(in) :: constraints
+     class(pdg_list_t), pointer :: pl
+     integer :: i
+     logical :: passed
+     type(vertex_iterator_t) :: vit
+     integer, dimension(:), allocatable :: pdg1, pdg2
+     integer :: n_emitters
+     integer, dimension(20) :: emitters_tmp
+
+     n_emitters = 0
+     pl => table%first
+     do i = 1, pl%get_size ()
+        call constraints%check_before_split (table, pl, i, passed)
+        if (passed) then
+           pdg1 = pl%get(i)
+           call vit%init (table%model, pdg1)
+           do 
+               call vit%get_next_match(pdg2)
+               if (allocated (pdg2)) then
+                  emitters_tmp (n_emitters+1) = pdg1(1)
+                  n_emitters = n_emitters + 1
+               else
+                  exit
+               end if
+           end do
+        end if
+     end do
+     allocate (emitters (n_emitters))
+     emitters = emitters_tmp (1:n_emitters)
+  end function ps_table_get_emitters
+
   subroutine ps_table_get_pdg_out (ps_table, i, pa_out, n_loop, n_rad)
     class(ps_table_t), intent(in) :: ps_table
     integer, intent(in) :: i
@@ -878,362 +905,4 @@ contains
   end subroutine ps_table_get_pdg_out
   
 
-  subroutine auto_components_test (u, results)
-    integer, intent(in) :: u
-    type(test_results_t), intent(inout) :: results
-    call test (auto_components_1, "auto_components_1", &
-         "generate decay table", &
-         u, results)
-    call test (auto_components_2, "auto_components_2", &
-         "generate NLO corrections, final state", &
-         u, results)
-    call test (auto_components_3, "auto_components_3", &
-         "generate NLO corrections, in and out", &
-         u, results)
-  end subroutine auto_components_test
-
-  subroutine auto_components_1 (u)
-    integer, intent(in) :: u
-    class(model_data_t), pointer :: model
-    type(field_data_t), pointer :: prt
-    type(ds_table_t) :: ds_table
-    type(split_constraints_t) :: constraints
-
-    write (u, "(A)")  "* Test output: auto_components_1"
-    write (u, "(A)")  "*   Purpose: determine Higgs decay table"
-    write (u, *)
-
-    write (u, "(A)")  "* Read Standard Model"
-
-    call prepare_model (model, var_str ("SM"))
-
-    prt => model%get_field_ptr (25)
-
-    write (u, *)
-    write (u, "(A)")  "* Higgs decays n = 2"
-    write (u, *)
-
-    call constraints%init (2)
-    call constraints%set (1, constrain_n_tot (2))
-    call constraints%set (2, constrain_mass_sum (prt%get_mass ()))
-
-    call ds_table%make (model, 25, constraints)
-    call ds_table%write (u)
-    call ds_table%final ()
-
-    write (u, *)
-    write (u, "(A)")  "* Higgs decays n = 3 (w/o radiative)"
-    write (u, *)
-
-    call constraints%init (3)
-    call constraints%set (1, constrain_n_tot (3))
-    call constraints%set (2, constrain_mass_sum (prt%get_mass ()))
-    call constraints%set (3, constrain_radiation ())
-
-    call ds_table%make (model, 25, constraints)
-    call ds_table%write (u)
-    call ds_table%final ()
-
-    write (u, *)
-    write (u, "(A)")  "* Higgs decays n = 3 (w/ radiative)"
-    write (u, *)
-
-    call constraints%init (2)
-    call constraints%set (1, constrain_n_tot (3))
-    call constraints%set (2, constrain_mass_sum (prt%get_mass ()))
-
-    call ds_table%make (model, 25, constraints)
-    call ds_table%write (u)
-    call ds_table%final ()
-
-    write (u, *)
-    write (u, "(A)")  "* Cleanup"
-    
-    call cleanup_model (model)
-
-    write (u, *)
-    write (u, "(A)")  "* Test output end: auto_components_1"
-
-  end subroutine auto_components_1
-
-  subroutine auto_components_2 (u)
-    integer, intent(in) :: u
-    class(model_data_t), pointer :: model
-    type(pdg_list_t), dimension(:), allocatable :: pl, pl_zzh
-    type(pdg_list_t) :: pl_match
-    type(fs_table_t) :: fs_table
-    type(split_constraints_t) :: constraints
-    real(default) :: sqrts
-    integer :: i
-
-    write (u, "(A)")  "* Test output: auto_components_2"
-    write (u, "(A)")  "*   Purpose: generate radiation (NLO)"
-    write (u, *)
-
-
-    write (u, "(A)")  "* Read Standard Model"
-
-    call prepare_model (model, var_str ("SM"))
-
-    write (u, *)
-    write (u, "(A)")  "* LO final state"
-    write (u, *)
-    
-    allocate (pl (2))
-    call pl(1)%init (2)
-    call pl(1)%set (1, 1)
-    call pl(1)%set (2, -1)
-    call pl(2)%init (2)
-    call pl(2)%set (1, 21)
-    call pl(2)%set (2, 21)
-    do i = 1, 2
-       call pl(i)%write (u);  write (u, *)
-    end do
-
-    write (u, *)
-    write (u, "(A)")  "* Initialize FS table"
-    write (u, *)
-    
-    call constraints%init (1)
-    call constraints%set (1, constrain_n_tot (3))
-
-    call fs_table%init (model, pl, constraints)
-    call fs_table%write (u)
-    
-    write (u, *)
-    write (u, "(A)")  "* Generate NLO corrections, unconstrained"
-    write (u, *)
-    
-    call fs_table%radiate (constraints)
-    call fs_table%write (u)
-    call fs_table%final ()
-    
-    write (u, *)
-    write (u, "(A)")  "* Generate NLO corrections, &
-         &complete but mass-constrained"
-    write (u, *)
-    
-    sqrts = 50
-
-    call constraints%init (2)
-    call constraints%set (1, constrain_n_tot (3))
-    call constraints%set (2, constrain_mass_sum (sqrts))
-
-    call fs_table%init (model, pl, constraints)
-    call fs_table%radiate (constraints)
-    call fs_table%write (u)
-    call fs_table%final ()
-    
-    write (u, *)
-    write (u, "(A)")  "* Generate NLO corrections, restricted"
-    write (u, *)
-    
-    call pl_match%init ([1, -1, 21])
-    
-    call constraints%init (2)
-    call constraints%set (1, constrain_n_tot (3))
-    call constraints%set (2, constrain_insert (pl_match))
-
-    call fs_table%init (model, pl, constraints)
-    call fs_table%radiate (constraints)
-    call fs_table%write (u)
-    call fs_table%final ()
-    
-    write (u, *)
-    write (u, "(A)")  "* Generate NNLO corrections, restricted, with one loop"
-    write (u, *)
-    
-    call pl_match%init ([1, -1, 21])
-    
-    call constraints%init (3)
-    call constraints%set (1, constrain_n_tot (4))
-    call constraints%set (2, constrain_n_loop (1))
-    call constraints%set (3, constrain_insert (pl_match))
-
-    call fs_table%init (model, pl, constraints)
-    call fs_table%enable_loops ()
-    call fs_table%radiate (constraints)
-    call fs_table%write (u)
-    call fs_table%final ()
-    
-    write (u, *)
-    write (u, "(A)")  "* Generate NNLO corrections, restricted, with loops"
-    write (u, *)
-    
-    call constraints%init (2)
-    call constraints%set (1, constrain_n_tot (4))
-    call constraints%set (2, constrain_insert (pl_match))
-
-    call fs_table%init (model, pl, constraints)
-    call fs_table%enable_loops ()
-    call fs_table%radiate (constraints)
-    call fs_table%write (u)
-    call fs_table%final ()
-    
-    write (u, *)
-    write (u, "(A)")  "* Generate NNLO corrections, restricted, to Z Z H, &
-         &no loops"
-    write (u, *)
-    
-    allocate (pl_zzh (1))
-    call pl_zzh(1)%init (3)
-    call pl_zzh(1)%set (1, 23)
-    call pl_zzh(1)%set (2, 23)
-    call pl_zzh(1)%set (3, 25)
-
-    call constraints%init (3)
-    call constraints%set (1, constrain_n_tot (5))
-    call constraints%set (2, constrain_mass_sum (500._default))
-    call constraints%set (3, constrain_require (pl_zzh(1)))
-
-    call fs_table%init (model, pl_zzh, constraints)
-    call fs_table%radiate (constraints)
-    call fs_table%write (u)
-    call fs_table%final ()
-    
-    call cleanup_model (model)
-
-    write (u, *)
-    write (u, "(A)")  "* Test output end: auto_components_2"
-
-  end subroutine auto_components_2
-
-  subroutine auto_components_3 (u)
-    integer, intent(in) :: u
-    class(model_data_t), pointer :: model
-    type(pdg_list_t), dimension(:), allocatable :: pl_in, pl_out
-    type(pdg_list_t) :: pl_match, pl_beam
-    type(if_table_t) :: if_table
-    type(split_constraints_t) :: constraints
-    real(default) :: sqrts
-    integer :: i
-
-    write (u, "(A)")  "* Test output: auto_components_3"
-    write (u, "(A)")  "*   Purpose: generate radiation (NLO)"
-    write (u, *)
-
-    write (u, "(A)")  "* Read Standard Model"
-
-    call prepare_model (model, var_str ("SM"))
-
-    write (u, *)
-    write (u, "(A)")  "* LO initial state"
-    write (u, *)
-    
-    allocate (pl_in (2))
-    call pl_in(1)%init (2)
-    call pl_in(1)%set (1, 1)
-    call pl_in(1)%set (2, -1)
-    call pl_in(2)%init (2)
-    call pl_in(2)%set (1, -1)
-    call pl_in(2)%set (2, 1)
-    do i = 1, 2
-       call pl_in(i)%write (u);  write (u, *)
-    end do
-
-    write (u, *)
-    write (u, "(A)")  "* LO final state"
-    write (u, *)
-    
-    allocate (pl_out (1))
-    call pl_out(1)%init (1)
-    call pl_out(1)%set (1, 23)
-    call pl_out(1)%write (u);  write (u, *)
-
-    write (u, *)
-    write (u, "(A)")  "* Initialize FS table"
-    write (u, *)
-    
-    call constraints%init (1)
-    call constraints%set (1, constrain_n_tot (4))
-    
-    call if_table%init (model, pl_in, pl_out, constraints)
-    call if_table%write (u)
-    
-    write (u, *)
-    write (u, "(A)")  "* Generate NLO corrections, unconstrained"
-    write (u, *)
-    
-    call if_table%radiate (constraints)
-    call if_table%write (u)
-    call if_table%final ()
-    
-    write (u, *)
-    write (u, "(A)")  "* Generate NLO corrections, &
-         &complete but mass-constrained"
-    write (u, *)
-    
-    sqrts = 100
-    call constraints%init (2)
-    call constraints%set (1, constrain_n_tot (4))
-    call constraints%set (2, constrain_mass_sum (sqrts))
-    
-    call if_table%init (model, pl_in, pl_out, constraints)
-    call if_table%radiate (constraints)
-    call if_table%write (u)
-    call if_table%final ()
-    
-    write (u, *)
-    write (u, "(A)")  "* Generate NLO corrections, &
-         &mass-constrained, restricted beams"
-    write (u, *)
-    
-    call pl_beam%init (3)
-    call pl_beam%set (1, 1)
-    call pl_beam%set (2, -1)
-    call pl_beam%set (3, 21)
-
-    call constraints%init (3)
-    call constraints%set (1, constrain_n_tot (4))
-    call constraints%set (2, constrain_in_state (pl_beam))
-    call constraints%set (3, constrain_mass_sum (sqrts))
-
-    call if_table%init (model, pl_in, pl_out, constraints)
-    call if_table%radiate (constraints)
-    call if_table%write (u)
-    call if_table%final ()
-    
-    write (u, *)
-    write (u, "(A)")  "* Generate NLO corrections, restricted"
-    write (u, *)
-    
-    call pl_match%init ([1, -1, 21])
-    
-    call constraints%init (4)
-    call constraints%set (1, constrain_n_tot (4))
-    call constraints%set (2, constrain_in_state (pl_beam))
-    call constraints%set (3, constrain_mass_sum (sqrts))
-    call constraints%set (4, constrain_insert (pl_match)) 
-
-    call if_table%init (model, pl_in, pl_out, constraints)
-    call if_table%radiate (constraints)
-    call if_table%write (u)
-    call if_table%final ()
-    
-    write (u, *)
-    write (u, "(A)")  "* Generate NNLO corrections, restricted, Z preserved, &
-         &with loops"
-    write (u, *)
-    
-    call constraints%init (5)
-    call constraints%set (1, constrain_n_tot (5))
-    call constraints%set (2, constrain_in_state (pl_beam))
-    call constraints%set (3, constrain_mass_sum (sqrts))
-    call constraints%set (4, constrain_insert (pl_match)) 
-    call constraints%set (5, constrain_require (pl_out(1)))
-
-    call if_table%init (model, pl_in, pl_out, constraints)
-    call if_table%enable_loops ()
-    call if_table%radiate (constraints)
-    call if_table%write (u)
-    call if_table%final ()
-    
-    call cleanup_model (model)
-
-    write (u, *)
-    write (u, "(A)")  "* Test output end: auto_components_3"
-
-  end subroutine auto_components_3
-
-
 end module auto_components
