@@ -1,4 +1,4 @@
-! WHIZARD 2.2.6 May 02 2015
+! WHIZARD 2.2.7 Aug 11 2015
 ! 
 ! Copyright (C) 1999-2015 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
@@ -38,15 +38,16 @@ module shower_base
   use constants
   use diagnostics
   use format_utils, only: write_separator
+  use lorentz
+  use particles
   use os_interface
   use rng_base
   use physics_defs
-  use lhapdf !NODEP!
-  use pdf_builtin !NODEP!
   use sm_physics, only: running_as_lam
   use particles
   use variables
   use model_data
+  use pdf
 
   implicit none
   private
@@ -55,14 +56,6 @@ module shower_base
   integer, parameter :: PS_PYTHIA6 = 2
   integer, parameter :: PS_PYTHIA8 = 3
   integer, parameter :: PS_UNDEFINED = 17
-  integer, parameter, public :: STRF_NONE = 0
-  integer, parameter, public :: STRF_LHAPDF6 = 1
-  integer, parameter, public :: STRF_LHAPDF5 = 2
-  integer, parameter, public :: STRF_PDF_BUILTIN = 3
-
-  logical, parameter, public :: DEBUG_WHIZ_SHOWER = .false.
-  logical, parameter, public :: ENSURE = .false.
-  logical, parameter, public :: DEBUG_SHOWER = .false.
   real(default), public :: D_min_scale = 0.5_default
   logical, public :: treat_light_quarks_massless = .true.
   logical, public :: treat_duscb_quarks_massless = .false.
@@ -72,10 +65,7 @@ module shower_base
   public :: PS_WHIZARD, PS_PYTHIA6, PS_PYTHIA8, PS_UNDEFINED
   public :: shower_method_of_string
   public :: shower_method_to_string
-  public :: pdf_data_t
   public :: shower_settings_t
-  public :: matching_settings_t
-  public :: matching_data_t
   public :: shower_base_t
   public :: D_alpha_s_isr
   public :: D_alpha_s_fsr
@@ -83,17 +73,6 @@ module shower_base
   public :: mass_squared_type
   public :: number_of_flavors
 
-  type :: pdf_data_t
-    type(lhapdf_pdf_t) :: pdf
-    real(default) :: xmin, xmax, qmin, qmax
-    integer :: type = STRF_NONE
-    integer :: set = 0
-   contains
-     procedure :: init => pdf_data_init
-     procedure :: write => pdf_data_write
-     procedure :: evolve => pdf_data_evolve
-  end type pdf_data_t
-
   type :: shower_settings_t
      logical :: active = .false.
      logical :: isr_active = .false.
@@ -102,13 +81,14 @@ module shower_base
      logical :: verbose = .false.
      integer :: method = PS_UNDEFINED
      logical :: hadronization_active = .false.
+     logical :: hadron_collision = .false.
      logical :: mlm_matching = .false.
      logical :: ckkw_matching = .false.
      logical :: powheg_matching = .false.
      type(string_t) :: pythia6_pygive
      !!! values present in PYTHIA and WHIZARDs PS,
      !!! comments denote corresponding PYTHIA values
-     real(default) :: d_min_t = 1._default          ! PARJ(82)^2
+     real(default) :: min_virtuality = 1._default          ! PARJ(82)^2
      real(default) :: fsr_lambda = 0.29_default     ! PARP(72)
      real(default) :: isr_lambda = 0.29_default     ! PARP(61)
      integer :: max_n_flavors = 5                   ! MSTJ(45)
@@ -129,16 +109,6 @@ module shower_base
      procedure :: write => shower_settings_write
   end type shower_settings_t
 
-  type, abstract :: matching_settings_t
-   contains
-     procedure (matching_settings_init), deferred :: init
-     procedure (matching_settings_write), deferred :: write
-  end type matching_settings_t
-
-  type, abstract :: matching_data_t
-     logical :: is_hadron_collision = .false.
-  end type matching_data_t
-
   type, abstract :: shower_base_t
      class(rng_t), allocatable :: rng
      type(string_t) :: name
@@ -148,26 +118,15 @@ module shower_base
      procedure :: write_msg => shower_base_write_msg
      procedure :: import_rng => shower_base_import_rng
      procedure (shower_base_init), deferred :: init
+     procedure (shower_base_prepare_new_event), deferred :: prepare_new_event
+     procedure (shower_base_import_particle_set), deferred :: import_particle_set
      procedure (shower_base_generate_emissions), deferred :: generate_emissions
+     procedure (shower_base_make_particle_set), deferred :: make_particle_set
+     procedure (shower_base_get_final_colored_ME_momenta), deferred :: &
+          get_final_colored_ME_momenta
   end type shower_base_t
 
 
-  abstract interface
-     subroutine matching_settings_init (settings, var_list)
-       import
-       class(matching_settings_t), intent(out) :: settings
-       type(var_list_t), intent(in) :: var_list
-     end subroutine matching_settings_init
-  end interface
-
-  abstract interface
-     subroutine matching_settings_write (settings, unit)
-       import
-       class(matching_settings_t), intent(in) :: settings
-       integer, intent(in), optional :: unit
-     end subroutine matching_settings_write
-  end interface
-
   abstract interface
     subroutine shower_base_init (shower, settings, pdf_data)
       import
@@ -178,23 +137,53 @@ module shower_base
    end interface
 
   abstract interface
-    subroutine shower_base_generate_emissions (shower, particle_set, &
-       model, model_hadrons, os_data, matching_settings, data, &
-       valid, vetoed, number_of_emissions)
+     subroutine shower_base_prepare_new_event &
+            (shower)
+       import
+       class(shower_base_t), intent(inout) :: shower
+     end subroutine shower_base_prepare_new_event
+  end interface
+
+  abstract interface
+     subroutine shower_base_import_particle_set &
+            (shower, particle_set, os_data)
+       import
+       class(shower_base_t), target, intent(inout) :: shower
+       type(particle_set_t), intent(in) :: particle_set
+       type(os_data_t), intent(in) :: os_data
+     end subroutine shower_base_import_particle_set
+  end interface
+
+  abstract interface
+     subroutine shower_base_generate_emissions &
+            (shower, valid, number_of_emissions)
       import
       class(shower_base_t), intent(inout), target :: shower
-      type(particle_set_t), intent(inout) :: particle_set
-      class(model_data_t), intent(in), target :: model
-      class(model_data_t), intent(in), target :: model_hadrons
-      class(matching_settings_t), intent(in), allocatable :: matching_settings
-      class(matching_data_t), intent(inout), allocatable :: data
-      type(os_data_t), intent(in) :: os_data
-      logical, intent(inout) :: valid
-      logical, intent(inout) :: vetoed
+       logical, intent(out) :: valid
       integer, optional, intent(in) :: number_of_emissions
     end subroutine shower_base_generate_emissions
    end interface
 
+  abstract interface
+     subroutine shower_base_make_particle_set &
+         (shower, particle_set, model, model_hadrons)
+       import
+       class(shower_base_t), intent(in) :: shower
+       type(particle_set_t), intent(inout) :: particle_set
+       class(model_data_t), intent(in), target :: model
+       class(model_data_t), intent(in), target :: model_hadrons
+     end subroutine shower_base_make_particle_set
+  end interface
+
+  abstract interface
+     subroutine shower_base_get_final_colored_ME_momenta &
+            (shower, momenta)
+       import
+       class(shower_base_t), intent(in) :: shower
+       type(vector4_t), dimension(:), allocatable, intent(out) :: momenta
+     end subroutine shower_base_get_final_colored_ME_momenta
+  end interface
+
 
 contains
 
@@ -228,53 +217,6 @@ contains
     end select
   end function shower_method_to_string
 
-  subroutine pdf_data_init (pdf_data, pdf_data_in)
-    class(pdf_data_t), intent(out) :: pdf_data
-    type(pdf_data_t), target, intent(in) :: pdf_data_in
-    pdf_data%xmin = pdf_data_in%xmin
-    pdf_data%xmax = pdf_data_in%xmax
-    pdf_data%qmin = pdf_data_in%qmin
-    pdf_data%qmax = pdf_data_in%qmax
-    pdf_data%set = pdf_data_in%set
-    pdf_data%type = pdf_data_in%type
-    if (pdf_data%type == STRF_LHAPDF6) then
-       if (pdf_data_in%pdf%is_associated ()) then
-          call lhapdf_copy_pointer (pdf_data_in%pdf, pdf_data%pdf)
-       else
-          call msg_bug ('pdf_data_init: pdf_data%pdf was not associated!')
-       end if
-    end if
-  end subroutine pdf_data_init
-
-  subroutine pdf_data_write (pdf_data, unit)
-    class(pdf_data_t), intent(in) :: pdf_data
-    integer, intent(in), optional :: unit
-    integer :: u
-    u = given_output_unit (unit); if (u < 0) return
-    write (u, "(3x,A,I0)") "PDF set  = ", pdf_data%set
-    write (u, "(3x,A,I0)") "PDF type = ", pdf_data%type
-  end subroutine pdf_data_write
-
-  subroutine pdf_data_evolve (pdf_data, x, q_in, f)
-    class(pdf_data_t), intent(inout) :: pdf_data
-    real(double), intent(in) :: x, q_in
-    real(double), dimension(-6:6), intent(out) :: f
-    real(double) :: q
-    select case (pdf_data%type)
-    case (STRF_PDF_BUILTIN)
-       call pdf_evolve_LHAPDF (pdf_data%set, x, q_in, f)
-    case (STRF_LHAPDF6)
-       q = min (pdf_data%qmax, q_in)
-       q = max (pdf_data%qmin, q)
-       call pdf_data%pdf%evolve_pdfm (x, q, f)
-    case (STRF_LHAPDF5)
-       q = min (pdf_data%qmax, q_in)
-       q = max (pdf_data%qmin, q)
-       call evolvePDFM (pdf_data%set, x, q, f)
-    case default
-       call msg_fatal ("PDF function: unknown PDF method.")
-    end select
-  end subroutine pdf_data_evolve
   subroutine shower_settings_init (shower_settings, var_list)
     class(shower_settings_t), intent(out) :: shower_settings
     type(var_list_t), intent(in) :: var_list
@@ -309,7 +251,7 @@ contains
          var_list%get_lval (var_str ("?shower_verbose"))
     shower_settings%pythia6_pygive = &
          var_list%get_sval (var_str ("$ps_PYTHIA_PYGIVE"))
-    shower_settings%d_min_t = &
+    shower_settings%min_virtuality = &
          (var_list%get_rval (var_str ("ps_mass_cutoff"))**2)
     shower_settings%fsr_lambda = &
          var_list%get_rval (var_str ("ps_fsr_lambda"))
@@ -354,6 +296,8 @@ contains
          "ps_isr_active                = ", settings%isr_active
     write (u, "(3x,A,1x,L1)") &
          "ps_fsr_active                = ", settings%fsr_active
+    write (u, "(3x,A,1x,L1)") &
+         "muli_active                  = ", settings%muli_active
     write (u, "(1x,A)")  "General settings:"
     if (settings%isr_active .or. settings%fsr_active) then
        write (u, "(3x,A)") &
@@ -362,7 +306,8 @@ contains
        write (u, "(3x,A,1x,L1)") &
             "shower_verbose               = ", settings%verbose
        write (u, "(3x,A,ES19.12)") &
-            "ps_mass_cutoff               = ", sqrt (abs (settings%d_min_t))
+            "ps_mass_cutoff               = ", &
+            sqrt (abs (settings%min_virtuality))
        write (u, "(3x,A,1x,I1)") &
             "ps_max_n_flavors             = ", settings%max_n_flavors
     else
@@ -427,22 +372,22 @@ contains
   function D_alpha_s_isr (tin, settings) result (alpha_s)
     real(default), intent(in) :: tin
     type(shower_settings_t), intent(in) :: settings
-    real(default) :: d_min_t, d_constalpha_s, d_lambda_isr
+    real(default) :: min_virtuality, d_constalpha_s, d_lambda_isr
     integer :: d_nf
     real(default) :: t
     real(default) :: alpha_s
-    d_min_t = settings%d_min_t
+    min_virtuality = settings%min_virtuality
     d_lambda_isr = settings%isr_lambda
     d_constalpha_s = settings%fixed_alpha_s
     d_nf = settings%max_n_flavors
     if (settings%alpha_s_fudged) then
-       t = max (max (0.1_default * d_min_t, &
+       t = max (max (0.1_default * min_virtuality, &
                      1.1_default * d_lambda_isr**2), abs(tin))
     else
        t = abs(tin)
     end if
     if (settings%isr_alpha_s_running) then
-       alpha_s = running_as_lam (number_of_flavors(t, d_nf, d_min_t), &
+       alpha_s = running_as_lam (number_of_flavors(t, d_nf, min_virtuality), &
             sqrt(t), d_lambda_isr, 0)
     else
        alpha_s = d_constalpha_s
@@ -452,68 +397,67 @@ contains
   function D_alpha_s_fsr (tin, settings) result (alpha_s)
     real(default), intent(in) :: tin
     type(shower_settings_t), intent(in) :: settings
-    real(default) :: d_min_t, d_lambda_fsr, d_constalpha_s
+    real(default) :: min_virtuality, d_lambda_fsr, d_constalpha_s
     integer :: d_nf
     real(default) :: t
     real(default) :: alpha_s
-    d_min_t = settings%d_min_t
+    min_virtuality = settings%min_virtuality
     d_lambda_fsr = settings%fsr_lambda
     d_constalpha_s = settings%fixed_alpha_s
     d_nf = settings%max_n_flavors
     if (settings%alpha_s_fudged) then
-       t = max (max (0.1_default * d_min_t, &
+       t = max (max (0.1_default * min_virtuality, &
                      1.1_default * d_lambda_fsr**2), abs(tin))
     else
        t = abs(tin)
     end if
     if (settings%fsr_alpha_s_running) then
-       alpha_s = running_as_lam (number_of_flavors (t, d_nf, d_min_t), &
+       alpha_s = running_as_lam (number_of_flavors (t, d_nf, min_virtuality), &
             sqrt(t), d_lambda_fsr, 0)
     else
        alpha_s = d_constalpha_s
     end if
   end function D_alpha_s_fsr
 
-  elemental function mass_type (type) result (mass)
+  elemental function mass_type (type, m2_default) result (mass)
     integer, intent(in) :: type
+    real(default), intent(in) :: m2_default
     real(default) :: mass
-    mass = sqrt (mass_squared_type (type))
+    mass = sqrt (mass_squared_type (type, m2_default))
   end function mass_type
 
-  elemental function mass_squared_type (type) result (mass2)
+  elemental function mass_squared_type (type, m2_default) result (mass2)
     integer, intent(in) :: type
+    real(default), intent(in) :: m2_default
     real(default) :: mass2
-
     select case (abs (type))
-    case (1,2)
-       if (treat_light_quarks_massless .or. &
-            treat_duscb_quarks_massless) then
-          mass2 = zero
-       else
-          mass2 = 0.330_default**2
-       end if
-    case (3)
-       if (treat_duscb_quarks_massless) then
-          mass2 = zero
-       else
-          mass2 = 0.500_default**2
-       end if
-    case (4)
-       if (treat_duscb_quarks_massless) then
-          mass2 = zero
-       else
-          mass2 = 1.500_default**2
-       end if
-    case (5)
-       if (treat_duscb_quarks_massless) then
-          mass2 = zero
-       else
-          mass2 = 4.800_default**2
-       end if
-    case (6)
-       mass2 = 175.00_default**2
-    case (GLUON)
-       mass2 = zero
+    !!!    case (1,2)
+    !!!       if (treat_light_quarks_massless .or. &
+    !!!            treat_duscb_quarks_massless) then
+    !!!          mass2 = zero
+    !!!       else
+    !!!          mass2 = 0.330_default**2
+    !!!       end if
+    !!!    case (3)
+    !!!       if (treat_duscb_quarks_massless) then
+    !!!          mass2 = zero
+    !!!       else
+    !!!          mass2 = 0.500_default**2
+    !!!       end if
+    !!!    case (4)
+    !!!       if (treat_duscb_quarks_massless) then
+    !!!          mass2 = zero
+    !!!       else
+    !!!          mass2 = 1.500_default**2
+    !!!       end if
+    !!!    case (5)
+    !!!       if (treat_duscb_quarks_massless) then
+    !!!          mass2 = zero
+    !!!       else
+    !!!          mass2 = 4.800_default**2
+    !!!       end if
+    !!!    case (GLUON)
+    !!!       mass2 = zero
     case (NEUTRON)
        mass2 = 0.939565_default**2
     case (PROTON)
@@ -540,27 +484,25 @@ contains
        mass2 = 5.8152_default**2
     case (SIGMABPLUS)
        mass2 = 5.8078_default**2
-    case (UNDEFINED)
-       mass2 = zero
     case (BEAM_REMNANT)
        mass2 = zero !!! don't know how to handle the beamremnant
     case default
-       mass2 = zero
+       mass2 = m2_default
     end select
   end function mass_squared_type
 
-  elemental function number_of_flavors (t, d_nf, d_min_t) result (nr)
-    real(default), intent(in) :: t, d_min_t
+  elemental function number_of_flavors (t, d_nf, min_virtuality) result (nr)
+    real(default), intent(in) :: t, min_virtuality
     integer, intent(in) :: d_nf
     real(default) :: nr
     integer :: i
     nr = 0
-    if (t < d_min_t) return   ! arbitrary cut off
+    if (t < min_virtuality) return   ! arbitrary cut off
     ! TODO: do i = 1, min (max (3, d_nf), 6)
     do i = 1, min (3, d_nf)
     !!! to do: take heavier quarks(-> cuts on allowed costheta in g->qq)
     !!!        into account
-       if ((four * mass_squared_type (i) + d_min_t) < t ) then
+       if ((four * mass_squared_type (i, zero) + min_virtuality) < t ) then
           nr = i
        else
           exit
